import os, pygame
from pygame.locals import *

from obstacles import *
from warps import *
from enemies import *

#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('Data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

class Level:
    def __init__(self, level, screen, shadow):
        self.sprLights =        pygame.sprite.RenderPlain()
        self.sprEnemies =       pygame.sprite.RenderPlain()
        self.sprObstacles =     pygame.sprite.RenderPlain()
        self.boxCount =         0 # how many boxes to press in this level?
        self.sprWarps =         pygame.sprite.RenderPlain()
        self.levelExtra =       []
        self.loadLevel(level, shadow)
        self.displayLevel(screen)

    def loadLevel(self, level, shadow):
        fullname = os.path.join('Levels', level)
        openLevel = open(fullname, 'r')
        levelLights =       []
        levelObstacles =    []
        levelEnemies =      []
        levelWarps  =       []
        for i in range(0, 5):
            line = openLevel.readline().strip() # thanks evilmrhenry for the fix
            if line == "Lights":
                for j in range(0, 7):
                    levelLights.append(openLevel.readline())
            elif line == "Enemies":
                for j in range(0, 7):
                    levelEnemies.append(openLevel.readline())
            elif line == "Obstacles":
                for j in range(0, 7):
                    levelObstacles.append(openLevel.readline())
            elif line == "Warps":
                for j in range(0, 7):
                    levelWarps.append(openLevel.readline())
            elif line == "Extra":
                for j in range(0, 7):
                    self.levelExtra.append(openLevel.readline())
        self.loadLights(levelLights, shadow)
        self.loadEnemies(levelEnemies)
        self.loadObstacles(levelObstacles)
        self.loadWarps(levelWarps)
        openLevel.close()

    def loadLights(self, levelLights, shadow):
        lamp, null = load_image('lamp1.gif')
        for floors in range(0, 7):
            floorLights = levelLights[floors]
            for lights in range(0, 5):
                if int(floorLights[lights]) != 0:
                    shadow.blit(lamp, ((130*lights+int(floorLights[lights])*10)-65,floors*64+34))

    def loadEnemies(self, levelEnemies):
        for floors in range(0, 7):
            floorEnemies = levelEnemies[floors]
            for enemies in range(0, 5):
                if floorEnemies[enemies] != '0':
                    if floorEnemies[enemies] == 'A':
                        self.sprEnemies.add(AlienEnemy((128*enemies, floors*64+43), floors))

    def loadObstacles(self, levelObstacles):
        for floors in range(0, 7):
            floorObstacles = levelObstacles[floors]
            for obstacles in range(0, 5):
                if floorObstacles[obstacles] != '0':
                    if floorObstacles[obstacles] == 'c':
                        self.sprObstacles.add(ControlBox(((128*obstacles), floors*64+43)))
                        self.boxCount = self.boxCount + 1 # we have more boxes
                    elif floorObstacles[obstacles] == 'l':
                        self.sprObstacles.add(Ladder(((128*obstacles), floors*64+36), True))
                    elif floorObstacles[obstacles] == 'b':
                        self.sprObstacles.add(Ladder(((128*obstacles), floors*64+36), False))
                    elif floorObstacles[obstacles] == 'e':
                        self.sprObstacles.add(Exit(((128*obstacles), floors*64+43)))

    def loadWarps(self, levelWarps):
        """Designed to load our warps into the appropriate sprites """
        currentChar = 'A'
        xy = (0,0)
        ab = (0,0)
        for floors in range(0, 7):
            floorWarps = levelWarps[floors]
            for warps in range(0, 5):
                if floorWarps[warps] == currentChar:
                    xy = ((128*warps+23),(floors*64+42))
                    for floorsT in range(floors, 7):
                        floorWarpsT = levelWarps[floorsT]
                        for warpsT in range(0, 5):
                            if floorWarpsT[warpsT] == currentChar:
                                ab = ((128*warpsT+23),(floorsT*64+42))
                    self.sprWarps.add(Teleporter(xy, ab, currentChar, floors))
                    self.sprWarps.add(Teleporter(ab, xy, currentChar, floors))
                    currentChar = chr(ord(currentChar)+1) # ord returns the integer equivalent of char

    def loadExtra(self):
        null = 0

    def displayLevel(self, screen):
        self.sprWarps.draw(screen)
        self.sprObstacles.draw(screen)

    def checkMap(self, player, screen):
        for obstacles in pygame.sprite.spritecollide(player, self.sprObstacles, 0):
            action = (obstacles.name(), obstacles.activate())
            if action[0] == "controlBox":
                self.boxCount = self.boxCount - 1
                if (self.boxCount == 0):
                    for items in self.sprObstacles:
                        if items.name() == "myExit":
                            items.Complete()
                            break
                self.displayLevel(screen)
            if action[0] == "myExit" and self.boxCount == 0:
                action = (obstacles.name(), obstacles.activate())
            return action
        for warp in pygame.sprite.spritecollide(player, self.sprWarps, 0):
            action = ("warp", warp.teleportTo())
            return action
        return ("nothing", 0)

    def update(self, screen, player):
        self.sprEnemies.update(player)
        self.sprEnemies.draw(screen)
